"""This module defines symbolic constants that are used by the C API."""

HDWF_NONE = 0
"""A HDWF value representing a bad device handle."""

RESULT_SUCCESS = 1
"""This value is returned by all C API functions in case of success.
The return type of API functions used to be a boolean, with true indicating success.
"""
